local media = LibStub("LibSharedMedia-3.0")

function PWBRagebar:BuildGui()
		-- Ragebar frame setup
	RageFrame = CreateFrame("Frame", nil, UIParent)
	RageFrame:SetFrameStrata("BACKGROUND")
	RageFrame:SetBackdrop({bgFile = media:Fetch("background", "Blizzard Tooltip"), tile = false})
	RageFrame:SetPoint("BOTTOMLEFT", self.db.x, self.db.y)

	-- Ragebar frame movement
	RageFrame:RegisterForDrag('LeftButton')
	RageFrame:SetClampedToScreen(true)
	RageFrame:SetScript("OnDragStart", function(self) PWBRagebar:Dragstart(self) end )
	RageFrame:SetScript('OnDragStop', function(self) PWBRagebar:Dragstop(self) end )

	-- Ragebar setup
	RageBar = CreateFrame("StatusBar", nil, RageFrame)
	RageBar:SetOrientation(self.db.orientation)
	RageBar:SetMinMaxValues(0, UnitPowerMax("player"))
	RageBar:SetValue(UnitPower("player"))
	RageBar:SetAllPoints(RageFrame)

	-- Ragebar text setup
	RageBarText = RageBar:CreateFontString(nil, 'OVERLAY')
	RageBarText:SetPoint("CENTER", RageFrame, 0, 0);
	RageBarText:SetFont(media:Fetch("font", self.db.font), self.db.fontsize, "OUTLINE")
	RageBarText:SetText(UnitPower("player"))

	PWBRagebar:UpdateStyling()
	PWBRagebar:UpdateVisiblity()
end

function PWBRagebar:UpdateStyling()
	local style = nil

	if self.db.useDefaults then
		style = Protwarbar.db.profile.default.appearence
	else
		style = self.db
	end

	local scale = Protwarbar.db.profile.default.appearence.scale

	RageFrame:SetWidth(self.db.width * scale)
	RageFrame:SetHeight(self.db.height * scale)
	RageFrame:SetMovable(not Protwarbar.db.profile.locked)
	RageFrame:EnableMouse(not Protwarbar.db.profile.locked)
	RageFrame:SetPoint("BOTTOMLEFT", self.db.x, self.db.y)

	RageBar:SetStatusBarTexture(media:Fetch("statusbar", style.barTexture))
	RageBar:SetOrientation(style.orientation)
	RageBar:SetMinMaxValues(0, UnitPowerMax("player"))
	RageBar:SetReverseFill(style.reverse)
	RageBarText:SetFont(media:Fetch("font", style.font), style.fontsize, "OUTLINE")

	if Protwarbar.db.profile.locked then
		RageFrame:SetBackdropColor(style.barBackground[1], style.barBackground[2],style.barBackground[3], style.barBackground[4])
	else
		RageFrame:SetBackdropColor(Protwarbar.db.profile.default.appearence.unlockbg[1],
			Protwarbar.db.profile.default.appearence.unlockbg[2],
			Protwarbar.db.profile.default.appearence.unlockbg[3],
			Protwarbar.db.profile.default.appearence.unlockbg[4])
	end
end

function PWBRagebar:UpdateVisiblity()
	if self.db.enabled == true and Protwarbar:IsVisible() then
		RageFrame:Show()
	else
		RageFrame:Hide()
	end
end

function PWBRagebar:Dragstart(frame)
	if not Protwarbar.db.profile.locked then
		frame:StartMoving()
	end
end

function PWBRagebar:Dragstop(frame)
	frame:StopMovingOrSizing()

	self.db.x = RageFrame:GetLeft()
	self.db.y = RageFrame:GetBottom()
end